@@ -3,7 +3,7 @@
  * @author Robert Carnell
  * @copyright Copyright (c) 2013, Robert Carnell
  * 
- * @license <a href="http://www.gnu.org/licenses/lgpl.html">GNU Lesser General Public License (LGPL v3)</a>
+ * License: <a href="http://www.gnu.org/licenses/lgpl.html">GNU Lesser General Public License (LGPL v3)</a>
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -31,15 +31,14 @@ COrthogonalArray::COrthogonalArray()
 	m_nrow = 0;
 	m_ncol = 0;
 	m_q = 0;
+    m_return_code = SUCCESS_CHECK;
+    m_warning_msg = "";
+    m_randomClass = RUnif();
 }
 
 void COrthogonalArray::createGaloisField(int q)
 {
-	bool test = galoisfield::GF_getfield(q, m_gf) == SUCCESS_CHECK ? true : false;
-	if (!test)
-	{
-		throw std::runtime_error("Could not construct the Galois field"); // LCOV_EXCL_TEST
-	}
+    m_gf = GaloisField(q);
 }
 
 void COrthogonalArray::checkDesignMemory()
@@ -56,22 +55,20 @@ int COrthogonalArray::checkMaxColumns(int k, int maxColumns)
 	{
 		return maxColumns;
 	}
-	else if (k > maxColumns)
+
+    if (k > maxColumns)
 	{
-        std::ostringstream s;
-        s << "At most " << maxColumns << " columns are possible for the design.";
-		const std::string ss = s.str();
-		throw std::runtime_error(ss.c_str());
-	}
-	else
-	{
-		return k;
+        std::ostringstream msg;
+        msg << "At most " << maxColumns << " columns are possible for the design.";
+        ostringstream_runtime_error(msg);
 	}
+    
+    return k;
 }
 
 void COrthogonalArray::checkResult(int result, int nvalue, int * n)
 {
-	if (result)
+	if (result == SUCCESS_CHECK)
 	{
 		*n = nvalue;
 	}
@@ -90,7 +87,27 @@ void COrthogonalArray::addelkemp(int q, int k, int* n)
 	checkDesignMemory();
 	int result = oaconstruct::addelkemp(m_gf, m_A, k);
 	checkResult(result, 2*q*q, n);
-    m_q = q; m_ncol=k; m_nrow=*n;
+
+    if (k == 2 * q + 1)
+    {
+        std::ostringstream msg;
+        msg << "\n\tWarning: The Addelman-Kempthorne construction with ncol = 2q+1\n";
+        msg << "\thas a defect.  While it is still an OA(2q^2,2q+1,q,2),\n";
+        msg << "\tthere exist some pairs of rows that agree in three columns.\n";
+        msg << "\tThe final column in the array is involved in all of these\n";
+        msg << "\ttriple coincidences.\n";
+        m_warning_msg = msg.str();
+        m_return_code = WARNING_CHECK;
+    }
+    else
+    {
+        m_return_code = SUCCESS_CHECK;
+        m_warning_msg = "";
+    }
+
+    m_q = q; 
+    m_ncol = k; 
+    m_nrow = *n;
 }
 
 void COrthogonalArray::addelkemp3(int q, int k, int* n)
@@ -102,10 +119,14 @@ void COrthogonalArray::addelkemp3(int q, int k, int* n)
 	checkDesignMemory();
 	int result = oaaddelkemp::addelkemp3(m_gf, m_A, k);
 	checkResult(result, 2*q*q*q, n);
-    m_q = q; m_ncol=k; m_nrow=*n;
+    m_return_code = SUCCESS_CHECK;
+    m_warning_msg = "";
+    m_q = q; 
+    m_ncol = k; 
+    m_nrow = *n;
 }
 
-void COrthogonalArray::addelkempn(int akn, int q, int k, int* n) // LCOV_EXCL_START
+void COrthogonalArray::addelkempn(int akn, int q, int k, int* n)
 {
 	k = checkMaxColumns(k, 2*(primes::ipow(q,akn)-1)/(q-1) - 1); /*  2(q^3-1)/(q-1) - 1  */
 	createGaloisField(q);
@@ -114,8 +135,12 @@ void COrthogonalArray::addelkempn(int akn, int q, int k, int* n) // LCOV_EXCL_ST
 	checkDesignMemory();
 	int result = oaaddelkemp::addelkempn(m_gf, akn, m_A, k);
 	checkResult(result, 2*primes::ipow(q,akn), n);
-    m_q = q; m_ncol=k; m_nrow=*n;
-} // LCOV_EXCL_STOP
+    m_return_code = SUCCESS_CHECK;
+    m_warning_msg = "";
+    m_q = q;
+    m_ncol = k; 
+    m_nrow = *n;
+}
 
 void COrthogonalArray::bose(int q, int k, int* n)
 {
@@ -126,23 +151,46 @@ void COrthogonalArray::bose(int q, int k, int* n)
 	checkDesignMemory();
 	int result = oaconstruct::bose(m_gf, m_A, k);
 	checkResult(result, q*q, n);
-    m_q = q; m_ncol=k; m_nrow=*n;
+    m_return_code = SUCCESS_CHECK;
+    m_warning_msg = "";
+    m_q = q;
+    m_ncol = k; 
+    m_nrow = *n;
 }
 
 void COrthogonalArray::bosebush(int q, int k, int *n)
 {
-	if (q%2)
+	if (q % 2 != 0)
 	{
 		throw std::runtime_error("This implementation of Bose-Bush only works for a number of levels equal to a power of 2");
 	}
-	k = checkMaxColumns(k, 2*q);
-	createGaloisField(2*q);
+	k = checkMaxColumns(k, 2*q + 1);
+    int q_star = 2 * q;
+	createGaloisField(q_star);
 	int matrows = 2 * q * q;
     m_A = bclib::matrix<int>(matrows, k);
 	checkDesignMemory();
 	int result = oaconstruct::bosebush(m_gf, m_A, k);
 	checkResult(result, 2*q*q, n);
-    m_q = q; m_ncol=k; m_nrow=*n;
+
+    if (k == 2 * q + 1) 
+    {
+        std::ostringstream msg;
+        msg << "\n\tWarning: The Bose-Bush construction with ncol = 2q+1\n";
+        msg << "\thas a defect.  While it is still an OA(2q^2,2q+1,q,2),\n";
+        msg << "\tthere exist some pairs of rows that agree in three columns.\n\n";
+        m_warning_msg = msg.str();
+        m_return_code = WARNING_CHECK;
+    }
+    else
+    {
+        m_return_code = SUCCESS_CHECK;
+        m_warning_msg = "";
+    }
+
+    m_q = q; 
+    m_ncol = k; 
+    m_nrow = *n;
 }
 
 void COrthogonalArray::bosebushl(int lambda, int q, int k, int* n)
@@ -154,11 +202,11 @@ void COrthogonalArray::bosebushl(int lambda, int q, int k, int* n)
 	primes::primepow(lambda, &pl, &nl, &isppl);
 	primes::primepow(q , &pq, &nq, &isppq);
 
-	if (!isppq)
+	if (isppq == 0)
 	{
 		throw std::runtime_error("The Bose-Bush design requires that q be prime raised to a positive integral power.");
 	}
-	if (!isppl)
+	if (isppl == 0)
 	{
 		throw std::runtime_error("The Bose-Bush design requires that lambda be a prime raised to a positive integral power.");
 	}
@@ -173,7 +221,26 @@ void COrthogonalArray::bosebushl(int lambda, int q, int k, int* n)
 	checkDesignMemory();
 	int result = oaconstruct::bosebushl(m_gf, lambda, m_A, k);
 	checkResult(result, lambda*q*q, n);
-    m_q = q; m_ncol=k; m_nrow=*n;
+
+    if (k == lambda * q + 1)
+    {
+        std::ostringstream msg;
+        msg << "\n\tWarning: The Bose-Bush construction with ncol = lambda*q+1\n";
+        msg << "\thas a defect.  While it is still an OA(lambda*q^2,lambda*q+1,q,2),\n";
+        msg << "\tit may have worse coincidence properties than\n";
+        msg << "\tOA(lambda*q^2,lambda*q+1,q,2).\n";
+        m_warning_msg = msg.str();
+        m_return_code = WARNING_CHECK;
+    }
+    else
+    {
+        m_return_code = SUCCESS_CHECK;
+        m_warning_msg = "";
+    }
+
+    m_q = q; 
+    m_ncol = k; 
+    m_nrow = *n;
 }
 
 void COrthogonalArray::bush(int q, int k, int* n)
@@ -181,11 +248,30 @@ void COrthogonalArray::bush(int q, int k, int* n)
 	k = checkMaxColumns(k, q+1);
 	createGaloisField(q);
 	int matrows = q * q * q;
+    int str = 3;
     m_A = bclib::matrix<int>(matrows, k);
 	checkDesignMemory();
-	int result = oaconstruct::bush(m_gf, m_A, 3, k);
+	int result = oaconstruct::bush(m_gf, m_A, str, k);
 	checkResult(result, q*q*q, n);
-    m_q = q; m_ncol=k; m_nrow=*n;
+    if (str >= q + 1)
+    {
+        std::ostringstream msg;
+        msg << "\tBush's (1952) theorem has a condition t<q where t\n";
+        msg << "\tis the strength of the array and q is the number of symbols.\n";
+        msg << "\tHere we have t = " << str << " and q = " << q << ".  The array may still\n";
+        msg << "\tbe useful, but a full factorial would have at least as\n";
+        msg << "many columns.\n";
+        m_warning_msg = msg.str();
+        m_return_code = WARNING_CHECK;
+    }
+    else
+    {
+        m_return_code = SUCCESS_CHECK;
+        m_warning_msg = "";
+    }
+    m_q = q; 
+    m_ncol = k; 
+    m_nrow = *n;
 }
 
 void COrthogonalArray::busht(int str, int q, int k, int* n)
@@ -201,7 +287,11 @@ void COrthogonalArray::busht(int str, int q, int k, int* n)
 	checkDesignMemory();
 	int result = oaconstruct::bush(m_gf, m_A, str, k);
 	checkResult(result, primes::ipow(q,str), n);
-    m_q = q; m_ncol=k; m_nrow=*n;
+    m_return_code = SUCCESS_CHECK;
+    m_warning_msg = "";
+    m_q = q;
+    m_ncol = k; 
+    m_nrow = *n;
 }
 
 int COrthogonalArray::oaagree(bool verbose)
@@ -212,12 +302,12 @@ int COrthogonalArray::oaagree(bool verbose)
 
 	for (int i = 0; i < m_nrow; i++)
 	{
-	  for (int j = i+1; j < m_nrow; j++)
+	  for (int j = i + 1; j < m_nrow; j++)
 	  {
 		agree = 0;
 		for (int k = 0; k < m_ncol; k++)
 		{
-		  agree += (m_A(i,k) == m_A(j,k));
+		  agree += static_cast<int>(m_A(i, k) == m_A(j, k));
 		}
 		if (agree > maxagr)
 		{
@@ -230,7 +320,7 @@ int COrthogonalArray::oaagree(bool verbose)
 		  }
 		}
 	  }
-	  if (i && i % ROWCHECK == 0 && verbose)
+	  if (i != 0 && i % ROWCHECK == 0 && verbose)
       {
 		PRINT_OUTPUT << "Checked rows <= " << i << " vs all other rows.\n"; // LCOV_EXCL_LINE
       }
@@ -267,9 +357,9 @@ int COrthogonalArray::oatriple(bool verbose)
 				{
 					for (int i2 = i1+1; i2 < m_nrow; i2++)
 					{
-						a3 += ( m_A(i1,j1)==m_A(i2,j1) )&&( m_A(i1,j2)==m_A(i2,j2) )&&( m_A(i1,j3)==m_A(i2,j3) );
+						a3 += static_cast<int>((m_A(i1,j1)==m_A(i2,j1)) && (m_A(i1,j2)==m_A(i2,j2)) && (m_A(i1,j3)==m_A(i2,j3)));
 					}
-					if (a3)
+					if (a3 != 0)
 					{
                         if (verbose)
                         {
