@@ -3,7 +3,7 @@
  * @author Robert Carnell
  * @copyright Copyright (c) 2013, Robert Carnell
  * 
- * @license <a href="http://www.gnu.org/licenses/lgpl.html">GNU Lesser General Public License (LGPL v3)</a>
+ * License: <a href="http://www.gnu.org/licenses/lgpl.html">GNU Lesser General Public License (LGPL v3)</a>
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -26,8 +26,7 @@
 #define CORTHOGONALARRAY_H
 
 #include "OACommonDefines.h"
-#include "galdef.h"
-#include "galois.h"
+#include "GaloisField.h"
 #include "construct.h"
 #include "runif.h"
 #include "rutils.h"
@@ -171,11 +170,9 @@ namespace oacpp {
         /**
          * Construct an orthogonal array using the Addelman Kempthorne algorithm
          * 
-         * @deprecated This method is not included by default in Art Owens's
-         * project.  The method is in the code, but it is not compiled in
-         * the makefile.  Adding it as a target to the makefile creates a successful build,
-         * but running addelkempn 3 3 (which should be equivalent to addelkemp3 3)
-         * does not function in the MinGW64 build.
+         * This method is not included by default in Art Owens's
+         * project, but it is in the code.  It is not compiled in
+         * the makefile.  Adding it as a target to the makefile creates a successful build.
          * 
          * From the original documentation:
          * <blockquote>
@@ -185,6 +182,7 @@ namespace oacpp {
          * provides the example n=3,q=3 which is OA( 54,25,3,2 ).
          * Based on this example it is possible to make an educated
          * guess as to how the construction generalizes.
+         * </blockquote>
          * 
          * @param akn the exponent on <code>q</code> for the number of rows <code>n = 2q^akn</code>
          * @param q the number of symbols <code>(0,...,q-1)</code>
@@ -560,14 +558,26 @@ namespace oacpp {
          * @return the orthogonal array
          */
         const bclib::matrix<int> & getoa();
+        /**
+         * Get available warning message
+         * @return the message
+         */
+        const std::string getMessage();
+        /**
+         * Get the method return code
+         * @return the return code
+         */
+        int getReturnCode();
 
 	private:
-		struct GF m_gf; /**< Galois Field */
+        GaloisField m_gf; /**< Galois Field */
         bclib::matrix<int> m_A; /**< Orthogonal Array */
 		int m_nrow; /**< the number of rows in orthogonal array */
         int m_ncol; /**< the number of columns in the orthogonal array */
         int m_q; /**< the number of symbols in the orthogonal array */
-        RUnif m_randomClass;
+        RUnif m_randomClass; /**< a pseudo random number generator */
+        int m_return_code;
+        std::string m_warning_msg;
 
 		/**
          * Create a Galois Field
@@ -626,6 +636,17 @@ namespace oacpp {
     {
         return m_A;
     }
+    inline
+    int COrthogonalArray::getReturnCode()
+    {
+        return m_return_code;
+    }
+    inline
+    const std::string COrthogonalArray::getMessage()
+    {
+        return m_warning_msg;
+    }
+
 }
 
 #endif
