@@ -54,7 +54,7 @@ namespace oacpp
             } // LCOV_EXCL_STOP
         }
 
-        void OA_strength(int q, bclib::matrix<int> & A, int* str, int verbose)
+        void OA_strength(int q, const bclib::matrix<int> & A, int* str, int verbose)
         {
             *str = -1;
 
@@ -84,11 +84,9 @@ namespace oacpp
                 (*str)++;
                 test = OA_strt(q, A, *str + 1, verbose);
             }
-
-            return;
         }
 
-        int OA_str0(int q, bclib::matrix<int> & A, int verbose)
+        int OA_str0(int q, const bclib::matrix<int> & A, int verbose)
         {
             size_t nrow = A.rowsize();
             size_t ncol = A.colsize();
@@ -115,7 +113,7 @@ namespace oacpp
             return SUCCESS_CHECK;
         }
 
-        int OA_str1(int q, bclib::matrix<int> & A, int verbose)
+        int OA_str1(int q, const bclib::matrix<int> & A, int verbose)
         {
             size_t nrow = A.rowsize();
             size_t ncol = A.colsize();
@@ -129,7 +127,7 @@ namespace oacpp
                     PRINT_OUTPUT << "The array cannot have strength 1, because the number\n";
                     PRINT_OUTPUT << "of rows " << nrow << " is not a multiple of q = " << q << ".\n";
                 } // LCOV_EXCL_STOP
-                return 0;
+                return FAILURE_CHECK;
             }
 
             lambda = static_cast<int>(nrow) / q;
@@ -142,7 +140,7 @@ namespace oacpp
                     count = 0;
                     for (size_t i = 0; i < nrow; i++)
                     {
-                        count += (A(i,j1) == q1);
+                        count += static_cast<int>(A(i,j1) == q1);
                     }
                     if (count != lambda)
                     {
@@ -152,7 +150,7 @@ namespace oacpp
                             PRINT_OUTPUT << "the number of times A[," << j1 << "] = " << q1 << ".\n";
                             PRINT_OUTPUT << "This happened in " << count << " rows, it should have happened in " << lambda << " rows.\n";
                         } // LCOV_EXCL_STOP
-                        return 0;
+                        return FAILURE_CHECK;
                     }
                 }
                 if (work > MEDWORK && verbose > 0)
@@ -167,7 +165,7 @@ namespace oacpp
             return SUCCESS_CHECK;
         }
 
-        int OA_str2(int q, bclib::matrix<int> & A, int verbose)
+        int OA_str2(int q, const bclib::matrix<int> & A, int verbose)
         {
             size_t nrow = A.rowsize();
             size_t ncol = A.colsize();
@@ -182,16 +180,16 @@ namespace oacpp
                     PRINT_OUTPUT << "Array has only " << ncol << " column(s).  At least two\n";
                     PRINT_OUTPUT << "columns are necessary for strength 2 to make sense.\n";
                 } // LCOV_EXCL_STOP
-                return 0;
+                return FAILURE_CHECK;
             }
-            if (static_cast<int>(nrow) % (q * q))
+            if ((static_cast<int>(nrow) % (q * q)) != 0)
             {
                 if (verbose > 0) // LCOV_EXCL_START
                 {
                     PRINT_OUTPUT << "The array cannot have strength 2, because the number\n";
                     PRINT_OUTPUT << "of rows " << nrow << " is not a multiple of q^2 = " << q << "^2 = " << q*q << ".\n";
                 } // LCOV_EXCL_STOP
-                return 0;
+                return FAILURE_CHECK;
             }
 
             lambda = static_cast<int>(nrow) / (q * q);
@@ -209,7 +207,7 @@ namespace oacpp
                             count = 0;
                             for (size_t i = 0; i < nrow; i++)
                             {
-                                count += (A(i,j1) == q1) && (A(i,j2) == q2);
+                                count += static_cast<int>((A(i,j1) == q1) && (A(i,j2) == q2));
                             }
                             if (count != lambda)
                             {
@@ -219,7 +217,7 @@ namespace oacpp
                                     PRINT_OUTPUT << "the number of times (A[," << j1 << "],A[," << j2 << "]) = (" << q1 << "," << q2 << ").\n";
                                     PRINT_OUTPUT << "This happened in " << count << " rows, it should have happened in " << lambda << " rows.\n";
                                 } // LCOV_EXCL_STOP
-                                return 0;
+                                return FAILURE_CHECK;
                             }
                         }
                     }
@@ -234,10 +232,10 @@ namespace oacpp
             {
                 PRINT_OUTPUT << "The array has strength (at least) 2.\n"; // LCOV_EXCL_LINE
             }
-            return 1;
+            return SUCCESS_CHECK;
         }
 
-        int OA_str3(int q, bclib::matrix<int> & A, int verbose)
+        int OA_str3(int q, const bclib::matrix<int> & A, int verbose)
         {
             size_t nrow = A.rowsize();
             size_t ncol = A.colsize();
@@ -252,16 +250,16 @@ namespace oacpp
                     PRINT_OUTPUT << "Array has only " << ncol << " column(s).  At least three\n";
                     PRINT_OUTPUT << "columns are necessary for strength 3 to make sense.\n";
                 } // LCOV_EXCL_STOP
-                return 0;
+                return FAILURE_CHECK;
             }
-            if (static_cast<int>(nrow) % (q * q * q))
+            if (static_cast<int>(nrow) % (q * q * q) != 0)
             {
                 if (verbose > 0) // LCOV_EXCL_START
                 {
                     PRINT_OUTPUT << "The array cannot have strength 3, because the number\n";
                     PRINT_OUTPUT << "of rows " << nrow << " is not a multiple of q^3 = " << q << "^3 = " << q*q*q << ".\n";
                 } // LCOV_EXCL_STOP
-                return 0;
+                return FAILURE_CHECK;
             }
 
             lambda = static_cast<int>(nrow) / (q * q * q);
@@ -283,7 +281,7 @@ namespace oacpp
                                     count = 0;
                                     for (size_t i = 0; i < nrow; i++)
                                     {
-                                        count += (A(i,j1) == q1) && (A(i,j2) == q2) && (A(i,j3) == q3);
+                                        count += static_cast<int>((A(i,j1) == q1) && (A(i,j2) == q2) && (A(i,j3) == q3));
                                     }
                                     if (count != lambda)
                                     {
@@ -293,7 +291,7 @@ namespace oacpp
                                             PRINT_OUTPUT << "the number of times (A[," << j1 << "],A[," << j2 << "],A[," << j3 << "]) = (" << q1 << "," << q2 << "," << q3 << ").\n";
                                             PRINT_OUTPUT << "This happened in " << count << " rows, it should have happened in " << lambda << " rows.\n";
                                         } // LCOV_EXCL_STOP
-                                        return 0;
+                                        return FAILURE_CHECK;
                                     }
                                 }
                             }
@@ -309,10 +307,10 @@ namespace oacpp
             {
                 PRINT_OUTPUT << "The array has strength (at least) 3.\n"; // LCOV_EXCL_LINE
             }
-            return 1;
+            return SUCCESS_CHECK;
         }
 
-        int OA_str4(int q, bclib::matrix<int> & A, int verbose)
+        int OA_str4(int q, const bclib::matrix<int> & A, int verbose)
         {
             size_t nrow = A.rowsize();
             size_t ncol = A.colsize();
@@ -327,16 +325,16 @@ namespace oacpp
                     PRINT_OUTPUT << "Array has only " << ncol << " column(s).  At least four\n";
                     PRINT_OUTPUT << "columns are necessary for strength 4 to make sense.\n";
                 } // LCOV_EXCL_STOP
-                return 0;
+                return FAILURE_CHECK;
             }
-            if (static_cast<int>(nrow) % (q * q * q * q))
+            if (static_cast<int>(nrow) % (q * q * q * q) != 0)
             {
                 if (verbose > 0) // LCOV_EXCL_START
                 {
                     PRINT_OUTPUT << "The array cannot have strength 4, because the number\n";
                     PRINT_OUTPUT << "of rows " << nrow << " is not a multiple of q^4 = " << q << "^4 = " << q*q*q*q << ".\n";
                 } // LCOV_EXCL_STOP
-                return 0;
+                return FAILURE_CHECK;
             }
 
             lambda = static_cast<int>(nrow) / (q * q * q * q);
@@ -366,7 +364,7 @@ namespace oacpp
                                             count = 0;
                                             for (size_t i = 0; i < nrow; i++)
                                             {
-                                                count += (A(i,j1) == q1) && (A(i,j2) == q2) && (A(i,j3) == q3) && (A(i,j4) == q4);
+                                                count += static_cast<int>((A(i,j1) == q1) && (A(i,j2) == q2) && (A(i,j3) == q3) && (A(i,j4) == q4));
                                             }
                                             if (count != lambda)
                                             {
@@ -376,7 +374,7 @@ namespace oacpp
                                                     PRINT_OUTPUT << "the number of times (A[," << j1 << "],A[," << j2 << "],A[," << j3 << "],A[," << j4 << "]) = (" << q1 << "," << q2 << "," << q3 << "," << q4 << ").\n";
                                                     PRINT_OUTPUT << "This happened in " << count << " rows, it should have happened in " << lambda << " rows.\n";
                                                 } // LCOV_EXCL_STOP
-                                                return 0;
+                                                return FAILURE_CHECK;
                                             }
                                         }
                                     }
@@ -395,10 +393,10 @@ namespace oacpp
             {
                 PRINT_OUTPUT << "The array has strength (at least) 4.\n"; // LCOV_EXCL_LINE
             }
-            return 1;
+            return SUCCESS_CHECK;
         }
 
-        int OA_strt(int q, bclib::matrix<int> & A, int t, int verbose)
+        int OA_strt(int q, const bclib::matrix<int> & A, int t, int verbose)
         {
             size_t nrow = A.rowsize();
             size_t ncol = A.colsize();
@@ -414,7 +412,7 @@ namespace oacpp
                     PRINT_OUTPUT << "Don't know how to verify strength " << t << ".  It doesn't\n";
                     PRINT_OUTPUT << "make sense.\n";
                 } // LCOV_EXCL_STOP
-                return 0;
+                return FAILURE_CHECK;
             }
             if (ncol < static_cast<size_t>(t))
             {
@@ -423,20 +421,20 @@ namespace oacpp
                     PRINT_OUTPUT << "Array has only " << ncol << " column(s).  At least " << t << "\n";
                     PRINT_OUTPUT << "columns are necessary for strength " << t << " to make sense.\n";
                 } // LCOV_EXCL_STOP
-                return 0;
+                return FAILURE_CHECK;
             }
             if (t == 0)
             {
                 return OA_str0(q, A, verbose);
             }
-            if (nrow % primes::ipow(q, t))
+            if (nrow % primes::ipow(q, t) != 0)
             {
                 if (verbose > 0) // LCOV_EXCL_START
                 {
                     PRINT_OUTPUT << "The array cannot have strength " << t << ", because the number\n";
                     PRINT_OUTPUT << "of rows " << nrow << " is not a multiple of q^" << t << " = " << q << "^" << t << " = " << primes::ipow(q, t) << ".\n";
                 } // LCOV_EXCL_STOP
-                return 0;
+                return FAILURE_CHECK;
             }
 
             lambda = static_cast<int>(nrow) / primes::ipow(q, t);
@@ -469,9 +467,9 @@ namespace oacpp
                     for (size_t row = 0; row < nrow; row++)
                     {
                         match = 1;
-                        for (int i = 0; i < t && match; i++)
+                        for (int i = 0; i < t && match != 0; i++)
                         {
-                            match *= A(row,clist[i]) == qlist[i];
+                            match *= static_cast<int>(A(row,clist[i]) == qlist[i]);
                         }
                         count += match;
                     }
@@ -483,7 +481,8 @@ namespace oacpp
                             PRINT_OUTPUT << "the number of times (";
                             for (int i = 0; i < t; i++)
                             {
-                                PRINT_OUTPUT << "A(," << clist[i] << ")" << ((i == t - 1) ? ")" : ",");
+                                std::string temp = (i == t - 1) ? ")" : ","; // warning about decaying a pointer when used on next line
+                                PRINT_OUTPUT << "A(," << clist[i] << ")" << temp.c_str();
                             }
                             PRINT_OUTPUT << " = (";
                             for (int i = 0; i < t; i++)
@@ -492,12 +491,12 @@ namespace oacpp
                             }
                             PRINT_OUTPUT << "This happened in " << count << " rows, it should have happened in " << lambda << " rows.\n";
                         } // LCOV_EXCL_STOP
-                        return 0;
+                        return FAILURE_CHECK;
                     }
                     for (int i = t - 1; i >= 0; i--) // has to be int
                     {
                         qlist[i] = (qlist[i] + 1) % q;
-                        if (qlist[i])
+                        if (qlist[i] != 0)
                         {
                             break;
                         }
@@ -507,7 +506,7 @@ namespace oacpp
                 for (int i = t - 1; i >= 0; i--) // has to be int
                 {
                     clist[i] = (clist[i] + 1) % (static_cast<int>(ncol) + i - t + 1);
-                    if (clist[i])
+                    if (clist[i] != 0)
                     {
                         break;
                     }
